/*
 * FOLLOW FOCUS SERVO V.1.0 - 05/01/2016
 * ------------------------------------------------
 * By Mirco Gaggiottini
 * 
 * HW:
 * 2 Button , 1 Servo , 1 Encoder (>=128), 1 Potentiometer (10K) , 1 Led (R-G) , 1 Arduino mini
 * Pinout:
 * 2 Encoder CH.A
 * 3 Encoder CH.B
 * 4 Button A
 * 5 Button B
 * 6 Led (Green)
 * 7 Led (Red)
 * 8 reverse switch
 * 9 servo focus
 * A0 Pot. Speed autofocus A-B
 */

#include <Servo.h>
//encoder
#define encoder0PinA 2
#define encoder0PinB 3
volatile int encoder0Count = 0;
volatile int oldCount = 0;
byte lastval[3]={0,0,0};
volatile float angle0 = 0;
volatile float velocity0 = 0;
volatile float angle0_previous = 0;
volatile float angle0_post = 0;

//Timer2
volatile int tcnt2 = 131;
volatile int t = 0;

//Servo
Servo myservo;

#define potvel A0      //Pot. speed control
#define aBUTTON 4      //A Button
#define bBUTTON 5      //B Button
#define ledG 6
#define ledR 7
#define reverseSW 8
#define servoFocus 9
int val = 710;
int vel = 0;
byte timepassed = 0;
byte mode=1;
int tick=1;
int oldval=0;
int oldread=0;
int posA = 0;
int posB = 0;
int posX = 0;
int pos = 0;
int oldpos = 0;
int btA =710;
int btB =710;
unsigned long currentMillis = 0;
boolean remote = false;
boolean busy = false;
byte oldcolor=0;

void setup() {
  // encoder
  pinMode(encoder0PinA, INPUT); 
  pinMode(encoder0PinB, INPUT); 
  attachInterrupt(0, doEncoder0A, CHANGE);  
  attachInterrupt(1, doEncoder0B, CHANGE);
    
  // setup for Timer2: interrupts every 1 ms
  TIMSK2 &= ~(1<<TOIE2);
  TCCR2A &= ~((1<<WGM21) | (1<<WGM20));
  TCCR2B &= ~(1<<WGM22);
  ASSR &= ~(1<<AS2);
  TIMSK2 &= ~(1<<OCIE2A);
  TCCR2B |= (1<<CS22)  | (1<<CS20);
  TCCR2B &= ~(1<<CS21);
  TCNT2 = tcnt2;
  TIMSK2 |= (1<<TOIE2);
  
  pinMode(aBUTTON,INPUT);
  pinMode(bBUTTON,INPUT);
  pinMode(reverseSW,INPUT);
  pinMode(ledG,OUTPUT);
  pinMode(ledR,OUTPUT);
  digitalWrite(ledG, HIGH);
  digitalWrite(aBUTTON, HIGH); //PULL-UP
  digitalWrite(bBUTTON, HIGH); //PULL-UP
  digitalWrite(reverseSW, HIGH); //PULL-UP
  myservo.attach(servoFocus);
  
  //start-up setup
  if (digitalRead(reverseSW) == LOW) {
    tick=-1;
  }
}

// Main ----------------------< 
void loop(){
  Button();
  readenc();
  if (mode==1) moveservo();
  if (oldpos!=pos) myservo.writeMicroseconds(pos);
  oldpos=pos;
}


//Functions -----------------------------------------<
void ledcolor(byte color) {
 switch (color) {
 case 0://off
 digitalWrite(ledG, LOW); digitalWrite(ledR, LOW);
 break;
 case 1://green
  digitalWrite(ledG, HIGH); digitalWrite(ledR, LOW);
 break;
 case 2://red
  digitalWrite(ledG, LOW); digitalWrite(ledR, HIGH);
 break;
 case 3://yellow
  digitalWrite(ledG, HIGH); digitalWrite(ledR, HIGH);
 break;
 }
oldcolor=color;
}

void gotopos(int posx) {
remote=true;
posX=posx;
int delta=abs(posx-pos);
ledcolor(2);
}


void readenc() {
  int readval = encoder0Count-oldCount; //absolute
  oldCount=encoder0Count;
  if (encoder0Count>32000 || encoder0Count<-32000) {encoder0Count=0;oldCount=0;}
  lastval[0]=lastval[1];
  lastval[1]=lastval[2];
  lastval[2]=readval;
  if (lastval[0]==1 && lastval[1]<1 && lastval[2]==1)readval=0;
  if (lastval[0]<1 && lastval[1]==1 && lastval[2]<1)readval=0;
  if (mode==1){
    //mode 1 - Auto + Speed
    if (abs(velocity0)<1)velocity0=1;
    val=val+(readval*abs(velocity0)); 
    if (val<710)val=710;
    if (val>2300)val=2300;

    //velocity
    vel = analogRead(potvel);
    vel = map(vel, 0, 1023, 2, 80);  
    
  } else {
    //mode 2 - Manual one revolution
    if (encoder0Count>500) encoder0Count=500;
    if (encoder0Count<0) encoder0Count=0;
    if (readval!=0) val= map((encoder0Count+oldval)/2,0,500,710,2300);
    val= map((encoder0Count+oldread)/2,0,500,710,2300);
    oldread=encoder0Count;
    if ((val<oldval+1)&&(val>oldval-1)) val=oldval;
    oldval=val;
    pos=val;
  }
  
  
}

void moveservo(){

  if (remote==true and ((val > oldval+5) or (val < oldval-5))) {
    remote=false;
    ledcolor(1);
  }
  
  if (remote==false){
  pos=val;
  oldval=pos;
  } else {

    if (pos!=posX) {
      busy=true;
    timepassed=timepassed+vel;
    if (millis()-currentMillis>100) {
      if (oldcolor==0) ledcolor(2); else ledcolor(0);
      currentMillis=millis(); 
      }
      
    if (timepassed>=100){
      timepassed=0;
      if (pos>posX) {
          pos=pos-1;
          val=pos;
          oldval=val;
          //if (pos<posX) pos=posX;
          } else {
          pos=pos+1;
          val=pos;
          oldval=val;
          //if (pos>posX) pos=posX;
          }

      }
    } else {
      busy=false;
      if (oldcolor==0) ledcolor(2);
    }
    }
}




